// ButtonsAndCountersFrame.java
//
// Informatics 45 Spring 2010
// Code Example: JPanels, Nested Layout, Model Separation, and Custom Events
// Version 2
//
// This version of ButtonsAndCountersFrame differs from the previous version
// in that the methods that built each of the four button panels have been
// combined together into a single method that takes parameters that control,
// for example, the name that should be placed on the button and the action
// listener that should be associated with it, so we don't have to write
// essentially the same code four times.  This is a reflex that it will be
// important for you to develop as you begin to write longer programs: when
// you find yourself repeating code that you already wrote (or nearly the
// same as what you already wrote), factoring out the commonalities and putting
// them in one place will make a dramatic difference in the maintainability of
// and readability of your program.

package inf45.spring2010.examples.gui2;

import java.awt.event.*;
import javax.swing.*;


public class ButtonsAndCountersFrame extends JFrame
{
	private int helloCount;
	private int goodbyeCount;
	private int booCount;

	private JLabel helloLabel;
	private JLabel goodbyeLabel;
	private JLabel booLabel;
	
	
	public ButtonsAndCountersFrame()
	{
		setTitle("Informatics 45 Spring 2009 - GUI Example #2");
		setSize(500, 150);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		helloCount = 0;
		goodbyeCount = 0;
		booCount = 0;
		
		buildUI();
	}

	
	// The four methods -- buildHelloPanel(), buildGoodbyePanel(),
	// buildBooPanel(), and buildEndPanel() -- have been rewritten as a
	// single method, buildButtonPanel(), with parameters introduced to
	// allow the method to do the right job in all four cases.  The
	// buildUI() method ends up needing to do a bit more work, since it
	// needs to provide those parameters; the payoff is that our four
	// nearly identical methods have been combined into a single method.
	
	private void buildUI()
	{
		BoxLayout mainLayout = new BoxLayout(getContentPane(), BoxLayout.X_AXIS);
		getContentPane().setLayout(mainLayout);

		getContentPane().add(Box.createHorizontalGlue());
		helloLabel = new JLabel();
		ActionListener helloActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					helloPressed();
				}
			};
		buildButtonPanel("Hello", helloLabel, helloActionListener);

		getContentPane().add(Box.createHorizontalStrut(10));
		goodbyeLabel = new JLabel();
		ActionListener goodbyeActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					goodbyePressed();
				}
			};
		buildButtonPanel("Goodbye", goodbyeLabel, goodbyeActionListener);

		getContentPane().add(Box.createHorizontalStrut(10));
		booLabel = new JLabel();
		ActionListener booActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					booPressed();
				}
			};
		buildButtonPanel("Boo", booLabel, booActionListener);
		
		getContentPane().add(Box.createHorizontalStrut(10));
		JLabel endLabel = new JLabel("!!!");
		ActionListener endActionListener =
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					endPressed();
				}
			};
		buildButtonPanel("End", endLabel, endActionListener);
		
		getContentPane().add(Box.createHorizontalGlue());
		
		updateCountLabels();
	}

	
	// In order to write this method so it does the right thing in all four
	// cases, there are three things we need to give it: the text that
	// should be placed on the button, the label that should be placed under
	// the button, and the action listener that should be associated with
	// the button.
	
	private void buildButtonPanel(
		String buttonText, JLabel label, ActionListener buttonActionListener)
	{
		JPanel buttonPanel = new JPanel();
		BoxLayout buttonPanelLayout = new BoxLayout(buttonPanel, BoxLayout.Y_AXIS);
		buttonPanel.setLayout(buttonPanelLayout);
		
		getContentPane().add(buttonPanel);
		
		JButton button = new JButton(buttonText);
		button.addActionListener(buttonActionListener);
		buttonPanel.add(button);

		buttonPanel.add(Box.createVerticalStrut(5));
		
		buttonPanel.add(label);
	}


	private void helloPressed()
	{
		helloCount++;
		updateCountLabels();
	}
	
	
	private void goodbyePressed()
	{
		goodbyeCount++;
		updateCountLabels();
	}
	
	
	private void booPressed()
	{
		booCount++;
		updateCountLabels();
	}
	
	
	private void endPressed()
	{
		dispose();
	}


	private void updateCountLabels()
	{
		helloLabel.setText(buildCountText(helloCount));
		goodbyeLabel.setText(buildCountText(goodbyeCount));
		booLabel.setText(buildCountText(booCount));
	}
	
	
	private String buildCountText(int count)
	{
		return "Count: " + count;
	}
}
