// Counters.java
//
// Informatics 45 Spring 2010
// Code Example: JPanels, Nested Layout, Model Separation, and Custom Events
// Version 4
//
// In this version of the Counters class, we've added one important missing
// piece to our model: the ability to tell the view when something has changed,
// so that the view can react appropriately (such as by changing the text in
// labels).
//
// We set up our notification scheme similarly to the notification schemes
// in Swing component like JButtons.  A Counters object contains a list of
// "counters changed listeners," who have registered an interest in being
// notified when the counters have changed.  It also provides two new methods,
// addCountersChangedListener() and removeCountersChangedListener(), which
// allow interested parties to register and cancel their interest in hearing
// when the counters have changed.  Finally, it also adds code that "fires"
// an event (by calling a method on each CountersChangedListener) whenever
// the counters are changed.
//
// This allows the model to keep the view notified of changes without having
// to know anything about the view, and also reduces the view's knowledge
// about the model (namely, knowing when it would be appropriate to ask the
// model for updated data to be redrawn).

package inf45.spring2010.examples.gui2;

import java.util.ArrayList;


public class Counters
{
	private int helloCount;
	private int goodbyeCount;
	private int booCount;
	private int endCount;
	
	private ArrayList<CountersChangedListener> listeners;
	
	
	public Counters()
	{
		helloCount = 0;
		goodbyeCount = 0;
		booCount = 0;
		endCount = 0;
		
		listeners = new ArrayList<CountersChangedListener>();
	}
	
	
	public int getHelloCount()
	{
		return helloCount;
	}
	
	
	public void incrementHelloCount()
	{
		helloCount++;
		fireCountersChanged();
	}
	
	
	public int getGoodbyeCount()
	{
		return goodbyeCount;
	}
	
	
	public void incrementGoodbyeCount()
	{
		goodbyeCount++;
		fireCountersChanged();
	}
	
	
	public int getBooCount()
	{
		return booCount;
	}
	
	
	public void incrementBooCount()
	{
		booCount++;
		fireCountersChanged();
	}
	
	
	public int getEndCount()
	{
		return endCount;
	}
	
	
	public void incrementEndCount()
	{
		endCount++;
		fireCountersChanged();
	}
	
	
	public void addCountersChangedListener(CountersChangedListener listener)
	{
		listeners.add(listener);
	}
	
	
	public void removeCountersChangedListener(CountersChangedListener listener)
	{
		listeners.remove(listener);
	}
	

	// This method "fires" a "counters changed" event by calling
	// countersChanged() on all the listeners.
	private void fireCountersChanged()
	{
		for (CountersChangedListener listener : listeners)
		{
			listener.countersChanged();
		}
	}
}
