// Counter.java
//
// Informatics 45 Spring 2010
// Code Example: JPanels, Nested Layout, Model Separation, and Custom Events
// Version 5
//
// This class implements a single counter, rather than a set of three counters.
// The advantage here is that we don't need three separate pairs of methods
// for getting and incrementing counters; instead, we need only write these
// methods once.

package inf45.spring2010.examples.gui2;

import java.util.ArrayList;


public class Counter
{
	private int count;
	private ArrayList<CounterChangedListener> listeners;
	
	
	public Counter()
	{
		count = 0;
		listeners = new ArrayList<CounterChangedListener>();
	}
	
	
	public int getCount()
	{
		return count;
	}
	
	
	public void incrementCount()
	{
		count++;
		fireCounterChanged();
	}
	
	
	public void addCounterChangedListener(CounterChangedListener listener)
	{
		listeners.add(listener);
	}
	
	
	public void removeCounterChangedListener(CounterChangedListener listener)
	{
		listeners.remove(listener);
	}


	private void fireCounterChanged()
	{
		for (CounterChangedListener listener : listeners)
		{
			listener.counterChanged();
		}
	}
}
